package com.muyu.etl.rule.domain.resp;

import com.muyu.etl.rule.domain.RuleInfo;
import com.muyu.etl.rule.domain.RuleVersion;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.util.List;
import java.util.function.Supplier;

/**
 * @Author WangXin
 * @Data 2024/8/22
 * @Description etl数据列表响应对象
 * @Version 1.0.0
 */
@Data
@SuperBuilder
@AllArgsConstructor
@NoArgsConstructor
@Tag(name = "规则列表响应对象",description = "查询规则列表信息需要输出的字段")
public class EtlRuleListResp {
    /** 主键 */
    @Schema(type = "Long", defaultValue = "1", description = "规则主键",requiredProperties = "true")
    private Long id;
    /** 规则名称 */
    @Schema(type = "String",defaultValue = "脱敏规则",description = "规则名称")
    private String ruleName;
    /** 规则编码 */
    @Schema(type = "String",defaultValue = "tel_desensitization_rule",description = "规则名称")
    private String ruleCode;
    /** 规则分类 */
    @Schema(type = "String",defaultValue = "system",description = "规则分类(字典字段)")
    private String ruleTypeValue;
    /** 规则说明 */
    @Schema(type = "String",defaultValue = "对手机号进行脱敏处理",description = "规则说明")
    private String ruleExplain;

    /** 规则状态(Y正常 N停用) */
    @Schema(type = "String",defaultValue = "Y",description = "规则状态(Y正常 N停用)")
    private String status;

    /** 规则级别 */
    @Schema(type = "String",defaultValue = "",description = "规则级别")
    private Long ruleLevelId;

    /** 是否激活（Y激活 N不激活） */
    @Schema(type = "String",defaultValue = "N",description = "是否激活（Y=激活,N=不激活）")
    private String isActivation;
    /** 规则版本列表 */
    @Schema(type = "List", description = "规则版本列表")
    private List<RuleVersion> ruleVersionList;

    public static EtlRuleListResp buildEtlRuleListResp(RuleInfo ruleInfo, List<RuleVersion> list) {
        return EtlRuleListResp.builder()
                .id(ruleInfo.getId())
                .ruleName(ruleInfo.getRuleName())
                .ruleCode(ruleInfo.getRuleCode())
                .ruleTypeValue(ruleInfo.getRuleTypeValue())
                .ruleExplain(ruleInfo.getRuleExplain())
                .status(ruleInfo.getStatus())
                .ruleLevelId(ruleInfo.getRuleLevelId())
                .isActivation(ruleInfo.getIsActivation())
                .ruleVersionList(list)
                .build();
    }

}
