package com.muyu.etl.rule.engine.core;



import com.muyu.etl.rule.engine.bean.annotation.EngineParam;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Author WangXin
 * @Data 2024/8/23
 * @Descriptio
 * @Version 1.0.0
 */

public class EngineContainer {

    private static final Logger log = LogManager.getLogger(EngineContainer.class.getName());

    public final static ConcurrentHashMap<String, Object> instanceMap = new ConcurrentHashMap<>(16);

    private final static ConcurrentHashMap<String, Class<?>> classMap = new ConcurrentHashMap<>(16);

    private final static ConcurrentHashMap<String, Method> methodMap = new ConcurrentHashMap<>(16);

    private final static ConcurrentHashMap<String, List<String>> methodEngineParamMap = new ConcurrentHashMap<>(16);

    public static void loadEngineInstance(String methodName, String engineKey, Class<?> aClass) {

        Method method = findByMethodByClass(methodName, aClass);

        List<String> engineParams = new ArrayList<>();

        try {
            instanceMap.put(engineKey, aClass.newInstance());
            log.info("[{}]类实例化成功---》class:{}",engineKey,aClass.getName());
        } catch (InstantiationException e) {
            log.error("[类实例化错误]--- {}", e.getMessage(), e);
            throw new RuntimeException(e);
        } catch (IllegalAccessException e) {
            log.error("[类实例化错误]--- {}", e.getMessage(), e);
            throw new RuntimeException(e);
        }

        classMap.put(engineKey, aClass);
        methodMap.put(engineKey, method);
        //获取方法所有参数类型
        Class<?>[] parameterTypes = method.getParameterTypes();
        if (parameterTypes.length == 0) {
            log.info("{}--》方法没有参数", methodName);
            return;
        }
        //获取方法参数注解
        Annotation[][] parameterAnnotations = method.getParameterAnnotations();
        //获取参数
        for (int i = 0; i < parameterTypes.length; i++) {
            Annotation[] parameterAnnotation = parameterAnnotations[i];
            if (parameterAnnotation.length == 0) {
                log.error("请给所有参数添加注解");
                throw new RuntimeException("请给所有参数添加注解");
            }
            if (parameterAnnotation.length > 1) {
                log.error("参数只能有一个注解");
                throw new RuntimeException("参数只能有一个注解");
            }
            EngineParam engineParam = null;
            if (parameterAnnotation[0] instanceof EngineParam) {
                engineParam = (EngineParam) parameterAnnotation[0];
            }
            if (engineParam == null) {
                throw new RuntimeException("请使用EngineParam注解");
            } else if (engineParam.name() != null && engineParam.name().isEmpty()) {
                throw new RuntimeException("EnginParam注解的name参数不能为空");
            }
            engineParams.add(engineParam.name());
        }
        methodEngineParamMap.put(engineKey, engineParams);
        log.info("初始化规则引擎成功---》规则引擎名称：{} ，执行方法：{}，参数：{}", engineKey, method.getName(), engineParams.toString());
    }

    /**
     * 通过方法名判断类中是否有此方法
     *
     * @param methodName 方法名
     * @param aClass     类的class对象
     * @return 如果有方法则返回此方法的method对象，没有则报错
     */
    private static Method findByMethodByClass(String methodName, Class<?> aClass) {
        Method method = null;
        for (Method aMethod : aClass.getMethods()) {
            if (aMethod.getName().equals(methodName)) {
                method = aMethod;
            }
        }
        if (method == null) {
            log.info("[{}] 类中没有 [{}] 方法", aClass.getName(), methodName);
            throw new RuntimeException("Method not found: " + methodName);
        } else {
            return method;
        }
    }

    public static Object getInstance(String engineKey) {
        return instanceMap.get(engineKey);
    }

    public static Method getMethodByClass(String engineKey) {
        return methodMap.get(engineKey);
    }

    public static List<String> getMethodEngineParam(String engineKey) {
        return methodEngineParamMap.get(engineKey);
    }

    public static Class<?> getClassByEngineKey(String engineKey) {
        return classMap.get(engineKey);
    }
}
