package com.muyu.etl.rule.engine.core;

import com.muyu.etl.rule.engine.core.EngineContainer;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Author WangXin
 * @Data 2024/8/23
 * @Description 规则引擎方法执行器
 * @Version 1.0.0
 */

public class EngineExecution {

     private static final Logger log = LogManager.getLogger(EngineExecution.class.getName());

    /**
     * 有参method执行器
     * @param engineKey 引擎名称
     * @param params 参数
     * @return 方法调用结果
     */
    public static Object argumentEngineExe(String engineKey, Map<String, Object> params){

        Object instance = EngineContainer.getInstance(engineKey);
        Method method = EngineContainer.getMethodByClass(engineKey);
        List<String> methodEngineParam = EngineContainer.getMethodEngineParam(engineKey);


        try {
            int methodParamLength = methodEngineParam.size();
            Object[] objects = new Object[methodParamLength];
            for (int i = 0; i < methodParamLength; i++) {
                objects[i] = params.get(methodEngineParam.get(i));
            }
            log.info("规则引擎 [{}] 调用参数 [{}]",engineKey, Arrays.toString(objects));
            Object invoke = method.invoke(instance, objects);
            log.info("规则引擎 [{}] 调用结果 [{}] ",engineKey, Arrays.toString(objects));
            return invoke;
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }
    /**
     * 有参method执行器
     * @param engineKey 引擎名称
     * @return 方法调用结果
     */
    public static Object noArgumentEngineExe(String engineKey){

        Object instance = EngineContainer.getInstance(engineKey);
        Method method = EngineContainer.getMethodByClass(engineKey);
        List<String> methodEngineParam = EngineContainer.getMethodEngineParam(engineKey);
        int methodParamLength = methodEngineParam.size();
        if (methodParamLength != 0){
            log.info("[方法名错误] 此方法为有参方法请选择 [argumentEngineExe] 方法输入对应参数");
            throw new RuntimeException("[方法名错误] 此方法为有参方法请选择 [argumentEngineExe] 方法输入对应参数");
        }
        try {
            log.info("规则引擎 [{}] 启动",engineKey);
            Object invoke = method.invoke(instance);
            log.info("规则引擎 [{}]  invoke 调用成功",engineKey);
            return invoke;
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }



}
