package com.muyu.etl.rule.engine.core.complier;

import com.muyu.etl.rule.engine.bean.constant.FileConstants;
import com.muyu.etl.rule.engine.util.FileLoadUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javax.tools.JavaCompiler;
import javax.tools.JavaFileObject;
import javax.tools.StandardJavaFileManager;
import javax.tools.ToolProvider;
import java.io.File;
import java.util.Arrays;
import java.util.List;

/**
 * @Author WangXin
 * @Data 2024/8/22
 * @Description java源码编译工具
 * @Version 1.0.0
 */
public class SourceCodeCompiler {

    private static final Logger log = LogManager.getLogger(SourceCodeCompiler.class.getName());
    /**
     * 获取系统java编译器
     */
    private static JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
    /**
     * 获取java文件管理器
     */
    private static StandardJavaFileManager fileManager = compiler.getStandardFileManager(null, null, null);

    private final static String workPath = System.getProperty("user.dir");

    /**
     * 输入编译的文件夹路径
     *
     * @param path 路径
     */
    public static void javaCompilerPath(String path) {
        List<File> javaFiles = FileLoadUtil.getJavaFiles(path);
        File[] array = javaFiles.toArray(new File[javaFiles.size()]);
        javaCompiler(path, array);
    }

    public static void javaCompilerFile(String... filePath) {
        int filePathLength = filePath.length;
        File[] files = new File[filePathLength];
        for (int i = 0; i < filePathLength; i++) {
            files[i] = new File(filePath[i]);
        }
        javaCompiler("filePath", files);
    }

    /**
     * 可同时编译多个java源代码
     *
     * @param file 文件
     */
    public static void javaCompiler(String path, File... file) {
        //通过源文件获取到要编译的Java源代码迭代器，包括所有内部类，其中每个类都是一个JavaFileObject,也被称为一个汇编单元
        // 使用文件对象管理器来获取文件
        Iterable<? extends JavaFileObject> javaFileObjects = fileManager.getJavaFileObjects(file);
        //生成编译任务
        log.info("程序运行地址为: {}", path);
        List<String> options = Arrays.asList("-classpath", path, "-verbose", "-d", path);
        JavaCompiler.CompilationTask task = compiler.getTask(null, fileManager, null, options, null, javaFileObjects);
        //执行编译任务
        Boolean call = task.call();

        if (call) {
            log.info("[{}个文件编译成功]", file.length);
        } else {
            throw new RuntimeException("[{}文件编译失败]： Compilation failed");
        }
    }

    /**
     * 可同时编译多个java源代码
     */
    public static void javaCompiler(String classConfigPath, String classpath, String javaPath, String engineKey) {
        //通过源文件获取到要编译的Java源代码迭代器，包括所有内部类，其中每个类都是一个JavaFileObject,也被称为一个汇编单元
        // 使用文件对象管理器来获取文件
        log.info("[开始编译] 请求参数 -----》 classConfigPath：{} --- classpath:{} --- javaPath:{} --- engineKey:{}", classConfigPath, classpath, javaPath, engineKey);
        String javaFilePath = javaPath + FileConstants.FILE_DELIMITER + engineKey + FileConstants.JAVA_SUFFIX;
        log.info("[java文件路径] ---> {}}", javaFilePath);
        // 构建编译参数数组
        String[] options = {
                "-classpath", classConfigPath,
                "-verbose",
                "-d", classpath,
                javaFilePath
        };
        log.info("[构建编译参数数组成功] ---> {}}", Arrays.toString(options));
        // 创建编译任务
        int run = compiler.run(null, null, null, options);
        // 关闭文件管理器

        if (run==0) {
            log.info("[文件编译成功]");
        } else {
            throw new RuntimeException(String.format("[{}文件编译失败]： Compilation failed",engineKey));
        }
    }
}
