package com.muyu.etl.rule.config;

import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @Author WangXin
 * @Data 2024/8/27
 * @Description 规则配置类
 * @Version 1.0.0
 */
@Configuration
@ConfigurationProperties(prefix = "engine")
@Data
public class EngineConfig {
    //执行方法名
    public String methodName;
    //操作地址
    public String sourceLocation;
    //编译地址
    public String classConfigPath;
    //包名
    public String packageName;
    //window地址源地址
    public String windowSourceLocation;
    //linux源地址
    public String linuxSourceLocation;
    //window class文件配置地址
    public String windowsClassConfigPath;
    //linux class文件配置地址
    public String linuxClassConfigPath;


    @Bean
    public EngineConfig initengineConfig() {
        //获取系统
        String osName = System.getProperty("os.name");
        if (osName.contains("Windows")) {
            sourceLocation = windowSourceLocation;
            classConfigPath = windowsClassConfigPath;
        } else if (osName.contains("Linux")) {
            sourceLocation = linuxSourceLocation;
            classConfigPath = linuxClassConfigPath;
        } else {
            throw new RuntimeException("暂无该系统");
        }
        return new EngineConfig();
    }

}
