package com.muyu.etl.rule.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.etl.rule.domain.RuleInfo;
import com.muyu.etl.rule.domain.req.EtlRuleListReq;
import com.muyu.etl.rule.domain.resp.EtlRuleListResp;
import com.muyu.etl.rule.service.IRuleInfoService;
import com.muyu.etl.rule.service.IRuleVersionService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.log4j.Log4j2;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author WangXin
 * @Data ${date}
 * @Description 规则维护控制层
 * @Version 1.0.0
 */
@RestController
@RequestMapping("/ruleInfo")
@Tag(name = "规则维护", description = "规则维护控制层")
@Log4j2
public class RuleInfoController extends BaseController {
    @Resource
    private IRuleInfoService ruleInfoService;
    @Resource
    private IRuleVersionService ruleVersionService;

    /**
     * 查询规则维护列表
     */
//  @RequiresPermissions("engine:ruleInfo:list")
    @PostMapping("/list")
    @Operation(summary = "查询规则维护列表", description = "根据规则维护请求参数查询规则维护列表")
    public Result<TableDataInfo<EtlRuleListResp>> list(@RequestBody EtlRuleListReq ruleInfoReq) {
        startPage();
        List<RuleInfo> list = ruleInfoService.selectRuleInfoList(ruleInfoReq);
        log.info("数据总条数---》{}条", list.size());
        return getDataTable(
                list.stream()
                        .map(ruleInfo -> EtlRuleListResp.buildEtlRuleListResp(
                                ruleInfo, ruleVersionService.selectRuleVersionListByRuleId(ruleInfo.getId())
                        ))
                        .collect(Collectors.toList())
        );
    }

    /**
     * 导出规则维护列表
     */
//  @RequiresPermissions("engine:ruleInfo:export")
    @PostMapping("/export")
    @Operation(summary = "导出规则维护列表", description = "根据规则维护请求参数导出规则维护列表")
    public void export(HttpServletResponse response, @RequestBody EtlRuleListReq ruleInfoReq) {
        List<RuleInfo> list = ruleInfoService.selectRuleInfoList(ruleInfoReq);
        ExcelUtil<RuleInfo> util = new ExcelUtil<RuleInfo>(RuleInfo.class);
        util.exportExcel(response, list, "规则维护数据");
    }

    /**
     * 获取规则维护详细信息
     */
//  @RequiresPermissions("engine:ruleInfo:query")
    @GetMapping(value = "/{id}")
    @Operation(summary = "获取规则维护详细信息", description = "根据规则维护id获取规则维护详细信息")
    public Result<EtlRuleListResp> getInfo(@PathVariable("id") Long id) {
        return success(EtlRuleListResp.buildEtlRuleListResp(
                ruleInfoService.selectRuleInfoById(id),
                ruleVersionService.selectRuleVersionListByRuleId(id)
        ));
    }

    /**
     * 新增规则维护
     */
//  @RequiresPermissions("engine:ruleInfo:add")
    @PostMapping
    @Operation(summary = "新增规则维护", description = "根据规则维护请求参数新增规则维护")
    public Result<Integer> add(
            @Validated @RequestBody RuleInfo ruleInfo) {
        if (ruleInfoService.checkIdUnique(ruleInfo)) {
            return error("新增 规则维护 '" + ruleInfo + "'失败，规则维护已存在");
        }
        ruleInfo.setCreateBy(SecurityUtils.getUsername());
        return toAjax(ruleInfoService.save(ruleInfo));
    }

    /**
     * 修改规则维护
     */
//  @RequiresPermissions("engine:ruleInfo:edit")
    @PutMapping
    @Operation(summary = "修改规则维护", description = "根据规则维护Id修改规则维护")
    public Result<Integer> edit(
            @Validated @RequestBody RuleInfo ruleInfo) {
        if (!ruleInfoService.checkIdUnique(ruleInfo)) {
            return error("修改 规则维护 '" + ruleInfo + "'失败，规则维护不存在");
        }
        ruleInfo.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(ruleInfoService.updateById(ruleInfo));
    }

    /**
     * 删除规则维护
     */
//  @RequiresPermissions("engine:ruleInfo:remove")
    @DeleteMapping("/{ids}")
    @Operation(summary = "删除规则维护", description = "根据规则维护Id删除规则维护")
    public Result<Integer> remove(@PathVariable("ids") Long[] ids) {
        ruleInfoService.removeBatchByIds(Arrays.asList(ids));
        return success();
    }
}
