package com.muyu.etl.rule.controller;

import java.util.Arrays;
import java.util.List;

import com.dtflys.forest.annotation.Post;
import com.muyu.etl.data.access.domain.DataAnalysis;
import com.muyu.etl.data.access.domain.resp.TableInfoResp;
import com.muyu.etl.domain.DataStructure;
import com.muyu.etl.rule.domain.RuleInfo;
import com.muyu.etl.rule.domain.req.DataTestReq;
import com.muyu.etl.rule.domain.req.ExecutionRuleVersionReq;
import com.muyu.etl.rule.domain.req.WriteFileReq;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;

import javax.annotation.Resource;

import org.apache.commons.codec.language.bm.RuleType;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.etl.rule.domain.RuleVersion;
import com.muyu.etl.rule.service.IRuleVersionService;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.security.utils.SecurityUtils;
import org.springframework.validation.annotation.Validated;
import com.muyu.common.core.web.page.TableDataInfo;

/**
 * @Author wangXin
 * @Data ${date}
 * @Description 规则版本控制层
 * @Version 1.0.0
 */
@RestController
@RequestMapping("/ruleVersion")
@Tag(name = "规则版本", description = "规则版本控制层")
public class RuleVersionController extends BaseController {
    @Resource
    private IRuleVersionService ruleVersionService;


    /**
     * 查询规则版本列表
     */
//  @RequiresPermissions("engine:ruleVersion:list")
    @PostMapping("/list")
    @Operation(summary = "查询规则版本列表", description = "根据规则版本请求参数查询规则版本列表")
    public Result<TableDataInfo<RuleVersion>> list(@RequestBody RuleVersion ruleVersion) {
        startPage();
        List<RuleVersion> list = ruleVersionService.selectRuleVersionList(ruleVersion);
        return getDataTable(list);
    }

    /**
     * 导出规则版本列表
     */
//  @RequiresPermissions("engine:ruleVersion:export")
    @PostMapping("/export")
    @Operation(summary = "导出规则版本列表", description = "根据规则版本请求参数导出规则版本列表")
    public void export(HttpServletResponse response, @RequestBody RuleVersion ruleVersion) {
        List<RuleVersion> list = ruleVersionService.selectRuleVersionList(ruleVersion);
        ExcelUtil<RuleVersion> util = new ExcelUtil<RuleVersion>(RuleVersion.class);
        util.exportExcel(response, list, "规则版本数据");
    }

    /**
     * 获取规则版本详细信息
     */
//  @RequiresPermissions("engine:ruleVersion:query")
    @GetMapping(value = "/{id}")
    @Operation(summary = "获取规则版本详细信息", description = "根据规则版本id获取规则版本详细信息")
    public Result<List<RuleVersion>> getInfo(@PathVariable("id") Long id) {
        return success(ruleVersionService.selectRuleVersionById(id));
    }

    /**
     * 新增规则版本
     */
//  @RequiresPermissions("engine:ruleVersion:add")
    @PostMapping
    @Operation(summary = "新增规则版本", description = "根据规则版本请求参数新增规则版本")
    @Transactional(rollbackFor = Exception.class)
    public Result<Integer> add(
            @Validated @RequestBody RuleVersion ruleVersion) {
        if (ruleVersionService.checkIdUnique(ruleVersion)) {
            return error("新增 规则版本 '" + ruleVersion + "'失败，规则版本已存在");
        }
        ruleVersion.setCreateBy(SecurityUtils.getUsername());
        boolean save = ruleVersionService.save(ruleVersion);
        if (save) {
            this.writeFile(
                    WriteFileReq.builder()
                            .fileName(ruleVersion.getRuleVersionClass())
                            .content(ruleVersion.getClassContent())
                            .build()
            );
        }
        return toAjax(save);
    }

    /**
     * 修改规则版本
     */
//  @RequiresPermissions("engine:ruleVersion:edit")
    @PutMapping
    @Operation(summary = "修改规则版本", description = "根据规则版本Id修改规则版本")
    public Result<Integer> edit(
            @Validated @RequestBody RuleVersion ruleVersion) {
        if (!ruleVersionService.checkIdUnique(ruleVersion)) {
            return error("修改 规则版本 '" + ruleVersion + "'失败，规则版本不存在");
        }
        ruleVersion.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(ruleVersionService.updateById(ruleVersion));
    }

    /**
     * 删除规则版本
     */
//  @RequiresPermissions("engine:ruleVersion:remove")
    @DeleteMapping("/{ids}")
    @Operation(summary = "删除规则版本", description = "根据规则版本Id删除规则版本")
    public Result<Integer> remove(@PathVariable("ids") Long[] ids) {
        ruleVersionService.removeBatchByIds(Arrays.asList(ids));
        return success();
    }

    /**
     * 根据文件名称获取规则文件内容
     *
     * @param fileName 文件名称
     * @return 文件内容
     */
//  @RequiresPermissions("engine:ruleVersion:readFile")
    @PostMapping("/readFile/{fileName}")
    @Operation(summary = "根据文件名称读取文件内容", description = "根据文件名称读取文件内容")
    public Result<String> readFile(@PathVariable("fileName") String fileName) {
        return Result.success(ruleVersionService.readFile(fileName));
    }

    /**
     * 根据文件名称写规则文件内容
     *
     * @param writeFileReq 文件写入请求对象
     * @return 结果集
     */
//  @RequiresPermissions("engine:ruleVersion:writeFile")
    @PostMapping("/writeFile")
    @Operation(summary = "根据文件名称写入文件内容", description = "根据文件名称写入文件内容")
    public Result<?> writeFile(@RequestBody WriteFileReq writeFileReq) {
        ruleVersionService.writeFile(writeFileReq);
        return Result.success();
    }

    /**
     * 执行java文件
     */
//  @RequiresPermissions("engine:ruleVersion:execution")
    @PostMapping("/execution")
    @Operation(summary = "执行java文件", description = "执行java文件")
    public Result<?> executionRuleVersion(@RequestBody ExecutionRuleVersionReq executionRuleVersionReq) {
        return Result.success(ruleVersionService.executionRuleVersion(executionRuleVersionReq));
    }

    /**
     * 执行值java文件
     */
//  @RequiresPermissions("engine:ruleVersion:execution")
    @PostMapping("/execution/{engineKey}")
    @Operation(summary = "执行值java文件", description = "执行值java文件")
    public Result<DataStructure> executionRuleVersion(@PathVariable("engineKey") String engineKey, @RequestBody DataStructure dataStructure) {
        return Result.success(ruleVersionService.executionRuleValueVersion(engineKey, dataStructure));
    }


    /**
     * 执行行java文件
     */
//  @RequiresPermissions("engine:ruleVersion:execution")
    @PostMapping("/lineFile/{engineKey}")
    @Operation(summary = "执行行java文件", description = "执行行java文件")
    public Result<DataStructure[][]> executionLineRuleVersion(@PathVariable("engineKey") String engineKey, @RequestBody DataStructure[][] dataStructureRow) {

        return Result.success(ruleVersionService.executionRowRuleVersion(engineKey, dataStructureRow));
    }

    /**
     * 执行组java文件
     */
//  @RequiresPermissions("engine:ruleVersion:execution")
    @PostMapping("/groupFile/{engineKey}")
    @Operation(summary = "执行组java文件", description = "执行组java文件")
    public Result<DataStructure[][]> executionGroupRuleVersion(@PathVariable("engineKey") String engineKey, @RequestBody DataStructure[][] dataStructureRows) {
        return Result.success(ruleVersionService.executionRowsRuleVersion(engineKey, dataStructureRows));
    }

    /**
     * 库表远调二级联动
     * @return 库表的二级联动
     */
    @GetMapping("/dataSource/tableInfo")
    @Operation(summary = "库表远调二级联动", description = "数据接入课表接入的二级联动，方便数据的选择")
    public Result<List<TableInfoResp>> dataSourceList() {
        return Result.success(ruleVersionService.dataSourceList());
    }

    /**
     * 获取表结构
     * @param id 表id
     * @return 数据表结构
     */
    @GetMapping("/dataSource/table/{id}")
    public Result<List<DataAnalysis>> dataSource(@PathVariable("id") Long id) {
        return Result.success(ruleVersionService.dataFindById(id));
    }

    /**
     * 获取表数据
     * @param dataTestReq 数据测试请求对象
     * @return ktlv数据结构
     */
    @PostMapping("/getEtlTableData")
    public Result<List<List<DataStructure>>> getEtlTableData(@RequestBody DataTestReq dataTestReq) {
        return Result.success(ruleVersionService.testData(dataTestReq));
    }

    @GetMapping("/isClassExist/{className}")
    public Result<Boolean> isClassExist(@PathVariable("className") String className) {
        return Result.success(ruleVersionService.isClassExist(className));
    }

    /**
     * 查詢所有規則(字段,数据集,记录)规则下拉框
     * @return List<RuleInfo>
     */
    @GetMapping("/getEtlRuleAll")
    public Result<List<RuleInfo>> getEtlRuleTypeByRuleId() {
        return Result.success(ruleVersionService.getEtlRuleTypeByRuleId());
    }

    /**
     * 根据规则id查询子规则类型
     * @param id
     * @return List<RuleVersion>
     */
    @GetMapping("/getEtlRuleTypeVById")
    public Result<List<RuleVersion>> getEtlRuleTypeVById(@RequestParam("id") Long id) {
        return Result.success(ruleVersionService.getEtlRuleTypeVById(id));
    }

    /**
     * 根据版本类型id查询 返回版本类型对象
     * @param id
     * @return RuleVersion
     */
    @GetMapping("/getETlRuleTypeVersionById")
    public Result<RuleVersion> getETlRuleTypeVersionById(@RequestParam("id") Long id) {
        return Result.success( ruleVersionService.getById(id));
    }


}
