package com.muyu.etl.rule.operation;

import com.alibaba.fastjson2.JSONObject;
import com.muyu.etl.domain.DataStructure;
import com.muyu.etl.enums.DataType;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.extern.log4j.Log4j2;

import java.util.Arrays;
import java.util.Iterator;

/**
 * @Author WangXin
 * @Data 2024/9/3
 * @Description 压栈工具
 * @Version 1.0.0
 */
@Data
@Builder
@AllArgsConstructor
@Log4j2
public class ArrayUtil<T> implements Iterator<T> {

    private final T[] originalArray; // 原始数组
    private T[] currentArray; // 当前数组，用于迭代
    private int currentIndex; // 当前索引
    private int lastReturnedIndex; // 上一次调用next()方法时返回的索引
    private boolean canRemove = false; // 标记是否可以移除元素

    public ArrayUtil(T[] array) {
        this.originalArray = array;
        this.currentArray = Arrays.copyOf(array, array.length);
        this.currentIndex = 0;
    }

    @Override
    public boolean hasNext() {
        return currentIndex < currentArray.length;
    }

    @Override
    public T next() {
        if (!hasNext()) {
            throw new IllegalStateException("No more elements");
        }
        lastReturnedIndex = currentIndex;
        canRemove = true; // 调用next()后可以调用remove()
        return currentArray[currentIndex++];
    }

    @Override
    public void remove() {
        if (!canRemove) {
            throw new IllegalStateException("Cannot call remove() without calling next()");
        }
        // 删除元素并更新数组
        System.arraycopy(currentArray, lastReturnedIndex + 1, currentArray, lastReturnedIndex, currentArray.length - lastReturnedIndex - 1);
        currentArray = Arrays.copyOf(currentArray, currentArray.length - 1); // 缩小数组大小
        currentIndex--; // 因为我们移除了当前元素，所以索引应该减一
        canRemove = false;
    }



        public static void main(String[] args) {
        DataStructure[] dataStructures = new DataStructure[6];
        dataStructures[0] = DataStructure.builder().value("aaa1").key("name").type(DataType.STRING).label("姓名").build();
        dataStructures[1] = DataStructure.builder().value("aaa2").key("name").type(DataType.STRING).label("姓名").build();
        dataStructures[2] = DataStructure.builder().value("aaa3").key("name").type(DataType.STRING).label("姓名").build();
        dataStructures[3] = DataStructure.builder().value("aaa4").key("name").type(DataType.STRING).label("姓名").build();
        dataStructures[4] = DataStructure.builder().value("aaa5").key("name").type(DataType.STRING).label("姓名").build();
        dataStructures[5] = DataStructure.builder().value("aaa6").key("name").type(DataType.STRING).label("姓名").build();

        ArrayUtil<DataStructure> dataStructureArrayUtil = new ArrayUtil<>(dataStructures);
        log.info(dataStructureArrayUtil.originalArray.length);
        log.info(JSONObject.toJSONString(dataStructureArrayUtil.originalArray));
        while (dataStructureArrayUtil.hasNext()){
            DataStructure next = dataStructureArrayUtil.next();
            if (next.getValue().equals("aaa3")){
                dataStructureArrayUtil.remove();
            }
        }
        log.info(dataStructureArrayUtil.currentArray.length);
        log.info(Arrays.toString(dataStructureArrayUtil.currentArray));

    }
}
