package com.muyu.etl.rule.service;

import java.util.List;

import com.muyu.common.core.domain.Result;
import com.muyu.etl.data.access.domain.DataAnalysis;
import com.muyu.etl.data.access.domain.resp.TableInfoResp;
import com.muyu.etl.domain.DataStructure;
import com.muyu.etl.rule.domain.RuleInfo;
import com.muyu.etl.rule.domain.RuleVersion;
import com.baomidou.mybatisplus.extension.service.IService;
import com.muyu.etl.rule.domain.req.DataTestReq;
import com.muyu.etl.rule.domain.req.ExecutionRuleVersionReq;
import com.muyu.etl.rule.domain.req.WriteFileReq;
import org.apache.commons.codec.language.bm.RuleType;

/**
 * 规则版本Service接口
 *
 * @author wangXin
 * @date 2024-08-25
 */
public interface IRuleVersionService extends IService<RuleVersion> {
    /**
     * 精确查询规则版本
     *
     * @param id 规则版本主键
     * @return 规则版本
     */
    public RuleVersion selectRuleVersionById(Long id);

    /**
     * 查询规则版本列表
     *
     * @param ruleVersion 规则版本
     * @return 规则版本集合
     */
    public List<RuleVersion> selectRuleVersionList(RuleVersion ruleVersion);

    /**
     * 判断 规则版本 id是否唯一
     * @param ruleVersion 规则版本
     * @return 结果
     */
    Boolean checkIdUnique(RuleVersion ruleVersion);

    /**
     * 通过规则id查询规则版本集合
     * @param id 规则id
     * @return 规则版本数据集
     */
    List<RuleVersion> selectRuleVersionListByRuleId(Long id);

    /**
     * 根据文件名称获取规则文件内容
     * @param fileName 文件名称
     * @return 文件内容
     */
    String readFile(String fileName);
    /**
     * 根据文件名称写入规则文件内容
     * @param  writeFileReq 文件名称
     */
    void writeFile(WriteFileReq writeFileReq);

    /**
     * 执行规则
     * @param executionRuleVersionReq 规则请求参数
     */
    Object executionRuleVersion(ExecutionRuleVersionReq executionRuleVersionReq);

    /**
     * 执行值规则
     * @param engineKey 规则类名
     * @param dataStructure 规则值处理数据
     */
    DataStructure executionRuleValueVersion(String engineKey, DataStructure dataStructure);

    /**
     * 执行值规则
     * @param engineKey 规则类名
     * @param dataStructureRow 规则行处理数据
     */
    DataStructure[][] executionRowRuleVersion(String engineKey, DataStructure[][] dataStructureRow);

    /**
     * 执行值规则
     * @param engineKey 规则类名
     * @param dataStructureRows 规则组处理数据
     */
    DataStructure[][] executionRowsRuleVersion(String engineKey, DataStructure[][] dataStructureRows);


    /**
     * 数据源列表
     * @return
     */
    List<TableInfoResp> dataSourceList();


    /**
     * 根据id查询数据表详情
     * @param id 表id
     * @return 表结构
     */
    List<DataAnalysis> dataFindById(Long id);

    /**
     * 测试数据
     * @param dataTestReq 测试请求参数
     * @return 测试结果
     */
    List<List<DataStructure>> testData(DataTestReq dataTestReq);

    /**
     *
     * @param className
     * @return
     */
    Boolean isClassExist(String className);
    /**
     * 查詢所有規則(字段,数据集,记录)规则下拉框
     * @return List<RuleInfo>
     */
    List<RuleInfo> getEtlRuleTypeByRuleId();


    List<RuleVersion> getEtlRuleTypeVById(Long id);


}
