package com.muyu.etl.rule.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.etl.rule.domain.RuleInfo;
import com.muyu.etl.rule.domain.req.EtlRuleListReq;
import com.muyu.etl.rule.mapper.RuleInfoMapper;
import com.muyu.etl.rule.service.IRuleInfoService;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.util.List;

/**
 * 规则维护Service业务层处理
 *
 * @author WangXin
 * @date 2024-08-25
 */
@Service
public class RuleInfoServiceImpl
        extends ServiceImpl<RuleInfoMapper, RuleInfo>
        implements IRuleInfoService {

    @Resource
    private RuleInfoMapper ruleInfoMapper;

    /**
     * 精确查询规则维护
     *
     * @param id 规则维护主键
     * @return 规则维护
     */
    @Override
    public RuleInfo selectRuleInfoById(Long id)
    {
        LambdaQueryWrapper<RuleInfo> queryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(id, "id不可为空");
        queryWrapper.eq(RuleInfo::getId, id);
        return this.getOne(queryWrapper);
    }


    /**
     * 查询规则维护列表
     *
     * @param ruleInfoReq 规则维护
     * @return 规则维护
     */
    @Override
    public List<RuleInfo> selectRuleInfoList(EtlRuleListReq ruleInfoReq)
    {
        return ruleInfoMapper.selectRuleInfoList(ruleInfoReq);
    }

    /**
     * 唯一 判断
     * @param ruleInfo 规则维护
     * @return 规则维护
    */
    @Override
    public Boolean checkIdUnique(RuleInfo ruleInfo) {
        LambdaQueryWrapper<RuleInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleInfo::getId, ruleInfo.getId());
        return this.count(queryWrapper) > 0;
    }

}
